@@ -1,25 +1,24 @@
 /* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2002 Ciaran Anscomb <evilwm@6809.org.uk>
+ * Copyright (C) 1999-2005 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
-#include "evilwm.h"
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <signal.h>
 #include <X11/cursorfont.h>
-#include <stdio.h>
-#ifdef SHAPE
-#include <X11/extensions/shape.h>
-#endif
+#include "evilwm.h"
+#include "log.h"
 
 Display		*dpy;
 int 		num_screens;
 ScreenInfo	*screens;
 ScreenInfo	*current_screen;
 Client		*current = NULL;
-Window		initialising = None;
+volatile Window	initialising = None;
 XFontStruct	*font;
-Client		*head_client;
+Client		*head_client = NULL;
+Application	*head_app = NULL;
 Atom		xa_wm_state;
 Atom		xa_wm_change_state;
 Atom		xa_wm_protos;
@@ -31,7 +30,7 @@ const char	*opt_display = "";
 const char	*opt_font = DEF_FONT;
 const char	*opt_fg = DEF_FG;
 const char	*opt_bg = DEF_BG;
-const char	**opt_term = NULL;
+const char	*opt_term[3] = { DEF_TERM, DEF_TERM, NULL };
 int		opt_bw = DEF_BW;
 #ifdef VWM
 const char	*opt_fc = DEF_FC;
@@ -48,24 +47,17 @@ int		quitting = 0;
 Atom		mwm_hints;
 #endif
 unsigned int numlockmask = 0;
+static unsigned int grabmask1 = ControlMask|Mod1Mask;
+/* This one is used for per-client mousebutton grabs, so global: */
+unsigned int grabmask2 = Mod1Mask;
 
+static void setup_display(void);
 static void *xmalloc(size_t size);
-
-static void *xmalloc(size_t size) {
-	void *ptr = malloc(size);
-	if (!ptr) {
-#ifdef STDIO
-		fprintf(stderr,"out of memory, looking for %d bytes\n",size);
-#endif
-		exit(1);
-	}
-	return ptr;
-}
+static unsigned int parse_modifiers(char *s);
 
 int main(int argc, char *argv[]) {
 	struct sigaction act;
 	int i;
-	XEvent ev;
 
 	for (i = 1; i < argc; i++) {
 		if (!strcmp(argv[i], "-fn") && i+1<argc)
@@ -84,31 +76,64 @@ int main(int argc, char *argv[]) {
 		else if (!strcmp(argv[i], "-bw") && i+1<argc)
 			opt_bw = atoi(argv[++i]);
 		else if (!strcmp(argv[i], "-term") && i+1<argc) {
-			opt_term = (const char **)xmalloc(3 * sizeof(const char *));
 			opt_term[0] = argv[++i];
 			opt_term[1] = opt_term[0];
-			opt_term[2] = NULL;
+#ifdef SNAP
 		} else if (!strcmp(argv[i], "-snap") && i+1<argc) {
 			opt_snap = atoi(argv[++i]);
+#endif
+		} else if (!strcmp(argv[i], "-app") && i+1<argc) {
+			Application *new = xmalloc(sizeof(Application));
+			char *tmp;
+			i++;
+			new->res_name = new->res_class = NULL;
+			new->geometry_mask = 0;
+#ifdef VWM
+			new->vdesk = -1;
+#endif
+			if ((tmp = strchr(argv[i], '/'))) {
+				*(tmp++) = 0;
+			}
+			if (strlen(argv[i]) > 0) {
+				new->res_name = xmalloc(strlen(argv[i])+1);
+				strcpy(new->res_name, argv[i]);
+			}
+			if (tmp && strlen(tmp) > 0) {
+				new->res_class = xmalloc(strlen(tmp)+1);
+				strcpy(new->res_class, tmp);
+			}
+			new->next = head_app;
+			head_app = new;
+		} else if (!strcmp(argv[i], "-g") && i+1<argc) {
+			i++;
+			if (!head_app)
+				continue;
+			head_app->geometry_mask = XParseGeometry(argv[i],
+					&head_app->x, &head_app->y,
+					&head_app->width, &head_app->height);
+#ifdef VWM
+		} else if (!strcmp(argv[i], "-v") && i+1<argc) {
+			i++;
+			if (head_app)
+				head_app->vdesk = atoi(argv[i]);
+#endif
+		} else if (!strcmp(argv[i], "-mask1") && i+1<argc) {
+			i++;
+			grabmask1 = parse_modifiers(argv[i]);
+		} else if (!strcmp(argv[i], "-mask2") && i+1<argc) {
+			i++;
+			grabmask2 = parse_modifiers(argv[i]);
 #ifdef STDIO
 		} else if (!strcmp(argv[i], "-V")) {
-			printf("evilwm version " VERSION "\n");
+			LOG_INFO("evilwm version " VERSION "\n");
 			exit(0);
 #endif
 		} else {
-#ifdef STDIO
-			printf("usage: evilwm [-display display] [-term termprog] [-fg foreground]\n");
-			printf("\t[-bg background] [-bw borderwidth] [-snap num] [-V]\n");
-#endif
+			LOG_INFO("usage: evilwm [-display display] [-term termprog] [-fg foreground]\n");
+			LOG_INFO("\t[-bg background] [-bw borderwidth] [-snap num] [-V]\n");
 			exit(2);
 		}
 	}
-	if (!opt_term) {
-		opt_term = (const char **)xmalloc(3 * sizeof(const char *));
-		opt_term[0] = DEF_TERM;
-		opt_term[1] = opt_term[0];
-		opt_term[2] = NULL;
-	}
 
 	act.sa_handler = handle_signal;
 	sigemptyset(&act.sa_mask);
@@ -119,52 +144,28 @@ int main(int argc, char *argv[]) {
 
 	setup_display();
 
-#ifdef SHAPE
-	have_shape = XShapeQueryExtension(dpy, &shape_event, &i);
-#endif
+	event_main_loop();
 
-	/* main event loop here */
-	for (;;) {
-		XNextEvent(dpy, &ev);
-		switch (ev.type) {
-			case KeyPress:
-				handle_key_event(&ev.xkey); break;
-#ifdef MOUSE
-			case ButtonPress:
-				handle_button_event(&ev.xbutton); break;
-#endif
-			case ConfigureRequest:
-				handle_configure_request(&ev.xconfigurerequest); break;
-			case MapRequest:
-				handle_map_request(&ev.xmaprequest); break;
-#ifdef VDESK
-			case ClientMessage:
-				handle_client_message(&ev.xclient); break;
-#endif
-#ifdef COLOURMAP
-			case ColormapNotify:
-				handle_colormap_change(&ev.xcolormap); break;
-#endif
-			case EnterNotify:
-				handle_enter_event(&ev.xcrossing); break;
-			case LeaveNotify:
-				handle_leave_event(&ev.xcrossing); break;
-			case PropertyNotify:
-				handle_property_change(&ev.xproperty); break;
-			case UnmapNotify:
-				handle_unmap_event(&ev.xunmap); break;
-			default:
-#ifdef SHAPE
-				if (have_shape && ev.type == shape_event) {
-					handle_shape_event((XShapeEvent *)&ev);
-				}
-#endif
-		}
-	}
 	return 1;
 }
 
-void setup_display() {
+static void *xmalloc(size_t size) {
+	void *ptr = malloc(size);
+	if (!ptr) {
+		/* C99 defines the 'z' printf modifier for variables of
+		 * type size_t.  Fall back to casting to unsigned long. */
+#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
+		LOG_ERROR("out of memory, looking for %zu bytes\n", size);
+#else
+		LOG_ERROR("out of memory, looking for %lu bytes\n",
+				(unsigned long)size);
+#endif
+		exit(1);
+	}
+	return ptr;
+}
+
+static void setup_display(void) {
 	XGCValues gv;
 	XSetWindowAttributes attr;
 	XColor dummy;
@@ -188,9 +189,7 @@ void setup_display() {
 
 	dpy = XOpenDisplay(opt_display);
 	if (!dpy) { 
-#ifdef STDIO
-		fprintf(stderr, "can't open display %s\n", opt_display);
-#endif
+		LOG_ERROR("can't open display %s\n", opt_display);
 		exit(1);
 	}
 	XSetErrorHandler(handle_xerror);
@@ -216,12 +215,10 @@ void setup_display() {
 	 * every combination of modifiers we can think of */
 	modmap = XGetModifierMapping(dpy);
 	for (i = 0; i < 8; i++) {
-		for (j = 0; j < modmap->max_keypermod; j++) {
+		for (j = 0; j < (unsigned int)modmap->max_keypermod; j++) {
 			if (modmap->modifiermap[i*modmap->max_keypermod+j] == XKeysymToKeycode(dpy, XK_Num_Lock)) {
 				numlockmask = (1<<i);
-#ifdef DEBUG
-				fprintf(stderr, "setup_display() : XK_Num_Lock is (1<<0x%02x)\n", i);
-#endif
+				LOG_DEBUG("setup_display() : XK_Num_Lock is (1<<0x%02x)\n", i);
 			}
 		}
 	}
@@ -234,7 +231,7 @@ void setup_display() {
 	gv.font = font->fid;
 
 	/* set up root window attributes - same for each screen */
-	attr.event_mask = ChildMask | PropertyChangeMask | EnterWindowMask | LeaveWindowMask
+	attr.event_mask = ChildMask | PropertyChangeMask | EnterWindowMask
 #ifdef COLOURMAP
 		| ColormapChangeMask
 #endif
@@ -243,20 +240,29 @@ void setup_display() {
 #endif
 		;
 
+	/* SHAPE extension? */
+#ifdef SHAPE
+	have_shape = XShapeQueryExtension(dpy, &shape_event, &i);
+#endif
+
 	/* now set up each screen in turn */
 	num_screens = ScreenCount(dpy);
-	screens = (ScreenInfo *)xmalloc(num_screens * sizeof(ScreenInfo));
-	for (i = 0; i < num_screens; i++) {
+	if (num_screens < 0) {
+		LOG_ERROR("Can't count screens\n");
+		exit(1);
+	}
+	screens = xmalloc(num_screens * sizeof(ScreenInfo));
+	for (i = 0; i < (unsigned int)num_screens; i++) {
 		char *ds, *colon, *dot;
 		ds = DisplayString(dpy);
 		/* set up DISPLAY environment variable to use */
-		colon = rindex(ds, ':');
+		colon = strrchr(ds, ':');
 		if (colon && num_screens > 1) {
-			screens[i].display = (char *)xmalloc(14 + strlen(ds));
+			screens[i].display = xmalloc(14 + strlen(ds));
 			strcpy(screens[i].display, "DISPLAY=");
 			strcat(screens[i].display, ds);
-			colon = rindex(screens[i].display, ':');
-			dot = index(colon, '.');
+			colon = strrchr(screens[i].display, ':');
+			dot = strchr(colon, '.');
 			if (!dot)
 				dot = colon + strlen(colon);
 			snprintf(dot, 5, ".%d", i);
@@ -280,18 +286,14 @@ void setup_display() {
 		/* So now I grab each and every one. */
 
 		for (keysym = keys_to_grab; *keysym; keysym++) {
-			grab_keysym(screens[i].root, ControlMask|Mod1Mask, *keysym);
+			grab_keysym(screens[i].root, grabmask1, *keysym);
 		}
-		grab_keysym(screens[i].root, Mod1Mask, XK_Tab);
+		grab_keysym(screens[i].root, grabmask2, KEY_NEXT);
 
 		/* scan all the windows on this screen */
-#ifdef XDEBUG
-		fprintf(stderr, "main:XQueryTree(); ");
-#endif
+		LOG_XDEBUG("main:XQueryTree(); ");
 		XQueryTree(dpy, screens[i].root, &dw1, &dw2, &wins, &nwins);
-#ifdef XDEBUG
-		fprintf(stderr, "%d windows\n", nwins);
-#endif
+		LOG_XDEBUG("%d windows\n", nwins);
 		for (j = 0; j < nwins; j++) {
 			XGetWindowAttributes(dpy, wins[j], &winattr);
 			if (!winattr.override_redirect && winattr.map_state == IsViewable)
@@ -301,3 +303,34 @@ void setup_display() {
 	}
 	current_screen = find_screen(DefaultScreen(dpy));
 }
+
+/* Used for overriding the default WM modifiers */
+static unsigned int parse_modifiers(char *s) {
+	static struct {
+		const char *name;
+		unsigned int mask;
+	} modifiers[9] = {
+		{ "shift", ShiftMask },
+		{ "lock", LockMask },
+		{ "control", ControlMask },
+		{ "alt", Mod1Mask },
+		{ "mod1", Mod1Mask },
+		{ "mod2", Mod2Mask },
+		{ "mod3", Mod3Mask },
+		{ "mod4", Mod4Mask },
+		{ "mod5", Mod5Mask }
+	};
+	char *tmp = strtok(s, ",+");
+	unsigned int ret = 0;
+	int i;
+	if (!tmp)
+		return 0;
+	do {
+		for (i = 0; i < 9; i++) {
+			if (!strcmp(modifiers[i].name, tmp))
+				ret |= modifiers[i].mask;
+		}
+		tmp = strtok(NULL, ",+");
+	} while (tmp);
+	return ret;
+}
