@@ -1,37 +1,25 @@
 /* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2002 Ciaran Anscomb <evilwm@6809.org.uk>
+ * Copyright (C) 1999-2005 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include "evilwm.h"
-
-#ifdef VWM
-# define HAS_HIDE 1
-#endif
-#ifdef VDESK
-# define HAS_HIDE 1
-#endif
+#include "log.h"
 
 #ifdef INFOBANNER
 Window info_window = None;
 
-void create_info_window(Client *c) {
+static void create_info_window(Client *c) {
 	char *name;
 	char buf[24];
 	int namew, iwinx, iwiny, iwinw, iwinh;
-	int width_inc = 1, height_inc = 1;
+	int width_inc = c->width_inc, height_inc = c->height_inc;
 
-#ifdef DEBUG
-	fprintf(stderr, "create_info_window() : Creating...\n");
-#endif
-	if (c->size->flags & PResizeInc) {
-		width_inc = c->size->width_inc;
-		height_inc = c->size->height_inc;
-	}
-	snprintf(buf, sizeof(buf), "%dx%d+%d+%d", c->width/width_inc,
-		c->height/height_inc, c->x, c->y);
+	LOG_DEBUG("create_info_window() : Creating...\n");
+	snprintf(buf, sizeof(buf), "%dx%d+%d+%d", (c->width-c->base_width)/width_inc,
+		(c->height-c->base_height)/height_inc, c->x, c->y);
 	iwinw = XTextWidth(font, buf, strlen(buf)) + 2;
 	iwinh = font->max_bounds.ascent + font->max_bounds.descent;
 	XFetchName(dpy, c->window, &name);
@@ -67,20 +55,18 @@ void create_info_window(Client *c) {
 		buf, strlen(buf));
 }
 
-void remove_info_window(void) {
-#ifdef DEBUG
-	fprintf(stderr, "remove_info_window() : Removing...\n");
-#endif
+static void remove_info_window(void) {
+	LOG_DEBUG("remove_info_window() : Removing...\n");
 	if (info_window)
 		XDestroyWindow(dpy, info_window);
 	info_window = None;
 }
 #endif  /* INFOBANNER */
 
-void draw_outline(Client *c) {
+static void draw_outline(Client *c) {
 #ifndef INFOBANNER_MOVERESIZE
 	char buf[24];
-	int width_inc = 1, height_inc = 1;
+	int width_inc = c->width_inc, height_inc = c->height_inc;
 #endif  /* ndef INFOBANNER */
 
 	XDrawRectangle(dpy, c->screen->root, c->screen->invert_gc,
@@ -88,12 +74,8 @@ void draw_outline(Client *c) {
 		c->width + c->border, c->height + c->border);
 
 #ifndef INFOBANNER_MOVERESIZE
-	if (c->size->flags & PResizeInc) {
-		width_inc = c->size->width_inc;
-		height_inc = c->size->height_inc;
-	}
-	snprintf(buf, sizeof(buf), "%dx%d+%d+%d", c->width/width_inc,
-			c->height/height_inc, c->x, c->y);
+	snprintf(buf, sizeof(buf), "%dx%d+%d+%d", (c->width-c->base_width)/width_inc,
+			(c->height-c->base_height)/height_inc, c->x, c->y);
 	XDrawString(dpy, c->screen->root, c->screen->invert_gc,
 		c->x + c->width - XTextWidth(font, buf, strlen(buf)) - SPACE,
 		c->y + c->height - SPACE,
@@ -112,30 +94,14 @@ void get_mouse_position(int *x, int *y, Window root) {
 #endif
 
 void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2) {
-	int basex, basey;
-
 	c->width = abs(x1 - x2);
 	c->height = abs(y1 - y2);
-
-	if (c->size->flags & PResizeInc) {
-		basex = (c->size->flags & PBaseSize) ? c->size->base_width :
-			(c->size->flags & PMinSize) ? c->size->min_width : 0;
-		basey = (c->size->flags & PBaseSize) ? c->size->base_height :
-			(c->size->flags & PMinSize) ? c->size->min_height : 0;
-		c->width -= (c->width - basex) % c->size->width_inc;
-		c->height -= (c->height - basey) % c->size->height_inc;
-	}
-
-	if (c->size->flags & PMinSize) {
-		if (c->width < c->size->min_width) c->width = c->size->min_width;
-		if (c->height < c->size->min_height) c->height = c->size->min_height;
-	}
-
-	if (c->size->flags & PMaxSize) {
-		if (c->width > c->size->max_width) c->width = c->size->max_width;
-		if (c->height > c->size->max_height) c->height = c->size->max_height;
-	}
-
+	c->width -= (c->width - c->base_width) % c->width_inc;
+	c->height -= (c->height - c->base_height) % c->height_inc;
+	if (c->min_width && c->width < c->min_width) c->width = c->min_width;
+	if (c->min_height && c->height < c->min_height) c->height = c->min_height;
+	if (c->max_width && c->width > c->max_width) c->width = c->max_width;
+	if (c->max_height && c->height > c->max_height) c->height = c->max_height;
 	c->x = (x1 <= x2) ? x1 : x1 - c->width;
 	c->y = (y1 <= y2) ? y1 : y1 - c->height;
 }
@@ -148,6 +114,7 @@ void sweep(Client *c) {
 
 	if (!grab_pointer(c->screen->root, MouseMask, resize_curs)) return;
 
+	XRaiseWindow(dpy, c->parent);
 #ifdef INFOBANNER_MOVERESIZE
 	create_info_window(c);
 #endif
@@ -179,6 +146,7 @@ void sweep(Client *c) {
 				remove_info_window();
 #endif
 				XUngrabPointer(dpy, CurrentTime);
+				moveresize(c);
 				return;
 		}
 	}
@@ -233,7 +201,11 @@ static void snap_client(Client *c) {
 	/* snap to other windows */
 	dx = dy = opt_snap;
 	for (ci = head_client; ci; ci = ci->next) {
-		if (ci != c && (ci->vdesk == vdesk || ci->vdesk == STICKY)) {
+		if (ci != c
+#ifdef VWM
+				&& (ci->vdesk == vdesk || ci->vdesk == STICKY)
+#endif
+				) {
 			if (ci->y - ci->border - c->border - c->height - c->y <= opt_snap && c->y - c->border - ci->border - ci->height - ci->y <= opt_snap) {
 				dx = absmin(dx, ci->x + ci->width - c->x + c->border + ci->border);
 				dx = absmin(dx, ci->x + ci->width - c->x - c->width);
@@ -263,6 +235,7 @@ void drag(Client *c) {
 	int reallymove = 0;
 
 	if (!grab_pointer(c->screen->root, MouseMask, move_curs)) return;
+	XRaiseWindow(dpy, c->parent);
 	get_mouse_position(&x1, &y1, c->screen->root);
 #ifdef INFOBANNER_MOVERESIZE
 	create_info_window(c);
@@ -315,10 +288,13 @@ void drag(Client *c) {
 				remove_info_window();
 #endif
 				XUngrabPointer(dpy, CurrentTime);
+#ifndef SOLIDDRAG
 				if (!reallymove) {
 					c->x = old_cx;
 					c->y = old_cy;
 				}
+				moveresize(c);
+#endif
 				return;
 			default:
 				break;
@@ -327,38 +303,17 @@ void drag(Client *c) {
 }
 #endif /* def MOUSE */
 
-void move(Client *c, int set) {
-	if (c) {
-		XRaiseWindow(dpy, c->parent);
-#ifdef MOUSE
-		if (!set)
-			drag(c);
-#endif
-		XMoveWindow(dpy, c->parent, c->x - c->border, c->y - c->border);
-		send_config(c);
-	}
-}
-
-void resize(Client *c, int set) {
-	if (c) {
-		XRaiseWindow(dpy, c->parent);
-#ifdef MOUSE
-		if (!set)
-			sweep(c);
-#endif
-		XMoveResizeWindow(dpy, c->parent, c->x - c->border,
-				c->y - c->border, c->width + (c->border*2),
-				c->height + (c->border*2));
-		XMoveResizeWindow(dpy, c->window, c->border, c->border,
-				c->width, c->height);
-		send_config(c);
-	}
+void moveresize(Client *c) {
+	XRaiseWindow(dpy, c->parent);
+	XMoveResizeWindow(dpy, c->parent, c->x - c->border,
+			c->y - c->border, c->width + (c->border*2),
+			c->height + (c->border*2));
+	XMoveResizeWindow(dpy, c->window, c->border, c->border,
+			c->width, c->height);
 }
 
 void maximise_horiz(Client *c) {
-#ifdef DEBUG
-	fprintf(stderr, "SCREEN: maximise_horiz()\n");
-#endif
+	LOG_DEBUG("SCREEN: maximise_horiz()\n");
 	if (c->oldw) {
 		c->x = c->oldx;
 		c->width = c->oldw;
@@ -372,9 +327,7 @@ void maximise_horiz(Client *c) {
 }
 
 void maximise_vert(Client *c) {
-#ifdef DEBUG
-	fprintf(stderr, "SCREEN: maximise_vert()\n");
-#endif
+	LOG_DEBUG("SCREEN: maximise_vert()\n");
 	if (c->oldh) {
 		c->y = c->oldy;
 		c->height = c->oldh;
@@ -387,82 +340,55 @@ void maximise_vert(Client *c) {
 	}
 }
 
-#ifdef HAS_HIDE
+#ifdef VWM
 void hide(Client *c) {
 	if (c) {
-		c->ignore_unmap += 2;
-#ifdef XDEBUG
-		fprintf(stderr, "screen:XUnmapWindow(parent); ");
-#endif
+		c->ignore_unmap++;
+		LOG_XDEBUG("screen:XUnmapWindow(parent); ");
 		XUnmapWindow(dpy, c->parent);
-#ifdef XDEBUG
-		fprintf(stderr, "screen:XUnmapWindow(window); ");
-#endif
-		XUnmapWindow(dpy, c->window);
 		set_wm_state(c, IconicState);
 	}
 }
 #endif
 
-void unhide(Client *c, int raise) {
+void unhide(Client *c, int raise_win) {
 	XMapWindow(dpy, c->window);
-	raise ? XMapRaised(dpy, c->parent) : XMapWindow(dpy, c->parent);
+	raise_win ? XMapRaised(dpy, c->parent) : XMapWindow(dpy, c->parent);
 	set_wm_state(c, NormalState);
 }
 
 void next(void) {
 	Client *newc = current;
-
-	if (!newc) {
-#ifdef DEBUG
-		fprintf(stderr,"NEXT: no current window, looking on this desktop\n");
-#endif
-		newc = head_client;
-#ifdef VWM
-		if(newc->vdesk != vdesk && newc->vdesk != STICKY) {
-			do {
-				newc = newc->next;
-			} while (newc && newc->vdesk != vdesk && newc->vdesk != STICKY);
-		}
-#endif
-	} else {
-#ifdef VWM
-		do {
-#endif
+	do {
+		if (newc) {
 			newc = newc->next;
-			if (current && !newc)
-				newc = head_client;
-#ifdef VWM
-		} while (newc && newc->vdesk != vdesk && newc->vdesk != STICKY);
-#endif
-	}
-	if (newc) {
-#ifdef VWM
-		if (newc->vdesk == vdesk || newc->vdesk == STICKY) {
-#endif
-			unhide(newc, RAISE);
-			setmouse(newc->window, 0, 0);
-			setmouse(newc->window, newc->width + newc->border - 1,
-				newc->height + newc->border - 1);
-#ifdef VWM
+			if (!newc && !current)
+				return;
 		}
-#endif
+		if (!newc)
+			newc = head_client;
+		if (!newc)
+			return;
+		if (newc == current)
+			return;
 	}
-#ifdef DEBUG
-       else {
-               fprintf(stderr,"NEXT: hmm, no next window\n");
-       }
+#ifdef VWM
+	while (newc->vdesk != vdesk && newc->vdesk != STICKY);
+#else
+	while (0);
 #endif
-
+	if (!newc)
+		return;
+	unhide(newc, RAISE);
+	select_client(newc);
+	setmouse(newc->window, 0, 0);
+	setmouse(newc->window, newc->width + newc->border - 1,
+		newc->height + newc->border - 1);
+	/* Need to think about this - see note about shaped
+	 * windows in TODO */
 }
 
 #ifdef VWM
-#ifdef VDESK_BOTH
-void switch_vdesk(int v) {
-	spawn_vdesk(v, NULL);
-	vdesk = v;
-}
-#else
 void switch_vdesk(int v) {
 	Client *c;
 	int wdesk;
@@ -478,9 +404,7 @@ void switch_vdesk(int v) {
 	if (current && current->vdesk != STICKY) {
 		client_update_current(current, NULL);
 	}
-#ifdef DEBUG
-	fprintf(stderr, "switch_vdesk() : Switching to desk %d\n", v);
-#endif
+	LOG_DEBUG("switch_vdesk() : Switching to desk %d\n", v);
 	for (c = head_client; c; c = c->next) {
 		wdesk = c->vdesk;
 		if (wdesk == vdesk) {
@@ -502,11 +426,8 @@ void switch_vdesk(int v) {
 		}
 	}
 	vdesk = v;
-#ifdef DEBUG
-	fprintf(stderr, "\tswitch_vdesk() : %d hidden, %d raised\n", hidden, raised);
-#endif
+	LOG_DEBUG("\tswitch_vdesk() : %d hidden, %d raised\n", hidden, raised);
 }
-#endif /* def VDESK_BOTH */
 #endif /* def VWM */
 
 ScreenInfo *find_screen(Window root) {
